aspect_ratios = {
	sixteen_nine = 9/16,
	four_three = 3/4
}

anim_configs = {}
anim_configs.doom_knuckles = {
	aspect_ratio = aspect_ratios.sixteen_nine,
	charge = {
		frames = {
			"guis/weapons/doom_knuckles/charge"
		},
		looping = true
	},
	attack = {
		frames = {
			"guis/weapons/doom_knuckles/attack_1",
			"guis/weapons/doom_knuckles/attack_2",
			"guis/weapons/doom_knuckles/attack_3",
			"guis/weapons/doom_knuckles/attack_2"
		},
		frame_interval = 0.1
	}
}
anim_configs.doom_chainsaw = {
	aspect_ratio = aspect_ratios.sixteen_nine,
	charge = {
		frames = {
			"guis/weapons/doom_chainsaw/charge"
		},
		looping = true
	},
	attack = {
		frames = {
			"guis/weapons/doom_chainsaw/attack_1",
			"guis/weapons/doom_chainsaw/attack_2"
		},
		frame_interval = 0.1
	}
}

TwoDWeaponGUI = TwoDWeaponGUI or class()

function TwoDWeaponGUI:init()
	local hud = managers.hud:script(PlayerBase.PLAYER_INFO_HUD_FULLSCREEN_PD2)
	local _ws = managers.hud._workspace

	self._panel = self._panel or hud and hud.panel or self._ws:panel({ name = "2DWeaponGUI" })

	self._weapon_bitmap = self._panel:bitmap({
		name = "weapon_bitmap",
		visible = false,
		valign = "bottom",
		layer = 0,
		color = Color.white
	})

	self._animation_start_time = Application:time()
	self._current_animation_length = 0
	self._current_animation_looping = false
end

function TwoDWeaponGUI:play_animation( weapon, anim )
	local hud = managers.hud:script(PlayerBase.PLAYER_INFO_HUD_FULLSCREEN_PD2)
	local width = hud.panel:w()
	local height = hud.panel:w() * ( anim_configs[weapon].aspect_ratio or 1 )

	self._weapon_bitmap:set_size( width, height )

	local y = hud.panel:h() - height
	self._weapon_bitmap:set_position( 0, y )

	self._animation_start_time = Application:time()

	self._weapon = weapon
	self._current_animation = anim
	local anim_config = anim_configs[weapon][anim]
	self._current_animation_length = #anim_config.frames * ( anim_config.frame_interval or 1 )
	self._current_animation_looping = anim_config.looping or false

	self._visible = true
	self._just_played_anim = true
end

function TwoDWeaponGUI:stop_animation()
	self._visible = false
end

function TwoDWeaponGUI:update( t )
	local player = managers.player:player_unit()
	if ( self._visible ) then
		self._weapon_bitmap:set_visible(true)

		if ( player ) then
			player:camera():camera_unit():set_visible(false)
			player:inventory():hide_equipped_unit()
		end

		if ( self._animation_start_time and self._weapon and self._current_animation ) then
			local t_change = t - self._animation_start_time;
			local anim_config = anim_configs[self._weapon][self._current_animation]
			local frames = anim_config.frames
			local frame_count = #frames

			local frame_interval = anim_config.frame_interval or 1
			local anim_length = self._current_animation_length
			local current_frame = math.ceil( ( (t_change%anim_length)/anim_length ) * frame_count )
			
			self._weapon_bitmap:set_image( frames[current_frame] )
		end

		if ( t > ( self._animation_start_time + self._current_animation_length ) ) and ( not self._current_animation_looping ) then
			self._visible = false
		end
	else
		if ( player ) and ( t > ( self._animation_start_time + self._current_animation_length ) + 0.2 ) and ( self._just_played_anim ) then
			self._weapon_bitmap:set_visible(false)
			self._weapon_bitmap:set_image( "guis/blank" )

			player:camera():camera_unit():set_visible(true)
			player:inventory():show_equipped_unit()
			self._just_played_anim = false
		end
	end
end

Hooks:PostHook(HUDManager, "init_finalize", "2DWeaponGUI", function(self)
	self._TwoDWeaponGUI = TwoDWeaponGUI:new()
end )

Hooks:PostHook(HUDManager, "update", "2DWeaponGUIUpdate", function(self, t, dt)
	self._TwoDWeaponGUI:update(t)
end )